DROP SCHEMA IF EXISTS  pgcv_core CASCADE;
CREATE SCHEMA pgcv_core;

-- =============================================
-- Author:      Roberto Mora
-- Description: N-dimentional array of int4 elements
-- =============================================
CREATE TYPE pgcv_core.ndarray_int4 AS (
  shape   int[],
  data    int[]
);
COMMENT ON TYPE pgcv_core.ndarray_int4 IS 'N-dimensional array of int4 elements. Used to represent and store images.';

-- =============================================
-- Author:      Roberto Mora
-- Description: Region properties of an object found in a binary image
-- =============================================
CREATE TYPE pgcv_core.regionprops AS (
  label         int,
  area          int,
  perimeter     float,
  centroid      float[2],
  solidity      float,
  eccentricity  float,
  convex_area   int,
  circularity   float,
  orientation   float,
  bbox          int[4]
);
COMMENT ON TYPE pgcv_core.regionprops IS 'Region properties of an object found in a binary image.';

-- =============================================
-- Author:      Roberto Mora
-- Description: Calculates the average hash of an image
-- =============================================
CREATE OR REPLACE FUNCTION pgcv_core.hash_avg(image pgcv_core.ndarray_int4, size int DEFAULT 8)
  RETURNS varchar
AS $$
"""
Calculates the average hash of an image.

Parameters
----------
image : ndarray
    The image represented by a pgcv_core.ndarray_int4.
size : int, optional
    Size of the hash. This size is used to resize the supplied image using 'lanczos' interpolation.
Returns
-------
hash : str
    The average hash in base 16.
Examples
--------
>>> SELECT pgcv_core.hash_avg(pgcv_io.image_read('/path/to/image.png'));
Notes
-----
The input image must be grayscale.
"""

import numpy as np
from scipy import misc

img = np.array(image["data"]).reshape(image["shape"]).astype('uint8')
pixels = misc.imresize(img, (size, size), 'lanczos')
avg = pixels.mean()
diff = pixels > avg

# Binary array to hex
h = 0
s = []
for i,v in enumerate(diff.flatten()):
  if v: h += 2**(i % 8)
  if (i % 8) == 7:
    s.append(hex(h)[2:].rjust(2, '0'))
    h = 0
return "".join(s)
$$ LANGUAGE plpython3u STRICT;
COMMENT ON FUNCTION pgcv_core.hash_avg(pgcv_core.ndarray_int4, int) IS 'Calculates the average hash of an image.';

-- =============================================
-- Author:      Roberto Mora
-- Description: Returns the thumbnail of an image with the specified width and height
-- =============================================
CREATE OR REPLACE FUNCTION pgcv_core.thumbnail(image pgcv_core.ndarray_int4, width int DEFAULT 200, height int DEFAULT 200)
  RETURNS pgcv_core.ndarray_int4
AS $$
"""
Returns the thumbnail of an image with the specified width and height.

Parameters
----------
image : ndarray
    The image represented by a pgcv_core.ndarray_int4.
width : int
    The target width of the thumbnail, it defaults to 200.
height : int
    The target height of the thumbnail, it defaults to 200.
Returns
-------
image : ndarray
    The thumbnail of the image of type pgcv_core.ndarray_int4.
Examples
--------
>>> SELECT shape from pgcv_core.thumbnail(pgcv_io.image_read('/path/to/image.png'), 250, 250);
"""

import numpy as np
from PIL import Image

img = np.array(image["data"]).reshape(image["shape"]).astype('uint8')
img = Image.fromarray(img)

size = width, height
img.thumbnail(size)

img = np.array(img)

return (list(img.shape), np.ravel(img))
$$ LANGUAGE plpython3u STRICT;
COMMENT ON FUNCTION pgcv_core.thumbnail(pgcv_core.ndarray_int4, int, int) IS 'Returns the thumbnail of an image with the specified width and height.';

-- =============================================
-- Author:      Roberto Mora
-- Description: Returns the data uri of the thumbnail of an image with the specified width and height, encoded in base64.
-- =============================================
CREATE OR REPLACE FUNCTION pgcv_core.thumbnail_uri_base64(image pgcv_core.ndarray_int4, width int DEFAULT 200, height int DEFAULT 200)
  RETURNS VARCHAR
AS $$
"""
Returns the data uri of the thumbnail of an image with the specified width and height, encoded in base64.

Parameters
----------
image : ndarray
    The image represented by a pgcv_core.ndarray_int4.
width : int
    The target width of the thumbnail, it defaults to 200.
height : int
    The target height of the thumbnail, it defaults to 200.
Returns
-------
image : ndarray
    The thumbnail of the image encoded in a base64 data uri.
Examples
--------
>>> SELECT shape from pgcv_core.thumbnail_uri_base64(pgcv_io.image_read('/path/to/image.png'), 250, 250);
"""

import numpy as np
from PIL import Image
import base64
from io import BytesIO

img = np.array(image["data"]).reshape(image["shape"]).astype('uint8')
img = Image.fromarray(img)

size = width, height
img.thumbnail(size)

buffered = BytesIO()
img.save(buffered, format="PNG")
img_str = base64.b64encode(buffered.getvalue())

return('data:image/png;base64,' + img_str.decode("utf-8"))
$$ LANGUAGE plpython3u STRICT;
COMMENT ON FUNCTION pgcv_core.thumbnail_uri_base64(pgcv_core.ndarray_int4, int, int) IS 'Returns the data uri of the thumbnail of an image with the specified width and height, encoded in base64.';

-- =============================================
-- Author:      Roberto Mora
-- Description: Returns the data uri of an image, encoded in base64.
-- =============================================
CREATE OR REPLACE FUNCTION pgcv_core.uri_base64(image pgcv_core.ndarray_int4)
  RETURNS VARCHAR
AS $$
"""
Returns the data uri of an image, encoded in base64.

Parameters
----------
image : ndarray
    The image represented by a pgcv_core.ndarray_int4.
Returns
-------
image : ndarray
    The image encoded in a base64 data uri.
Examples
--------
>>> SELECT shape from pgcv_core.uri_base64(pgcv_io.image_read('/path/to/image.png'));
"""

import numpy as np
from PIL import Image
import base64
from io import BytesIO

img = np.array(image["data"]).reshape(image["shape"]).astype('uint8')
img = Image.fromarray(img)

buffered = BytesIO()
img.save(buffered, format="PNG")
img_str = base64.b64encode(buffered.getvalue())

return('data:image/png;base64,' + img_str.decode("utf-8"))
$$ LANGUAGE plpython3u STRICT;
COMMENT ON FUNCTION pgcv_core.uri_base64(pgcv_core.ndarray_int4) IS 'Returns the data uri of an image, encoded in base64.';


-- =============================================
-- These queries test the core functions
-- =============================================
-- SELECT * FROM pgcv_core.hash_avg(pgcv_io.image_read('/Users/ro/U/[ Asistencia ] - Proyecto de Investigacion/Source_Images/mdb155.pgm'));
--
-- SELECT * from pgcv_core.thumbnail(pgcv_io.image_read('/Users/ro/U/[ Asistencia ] - Proyecto de Investigacion/Source_Images/mdb155.pgm'))
--
-- SELECT * from pgcv_core.thumbnail_uri_base64(pgcv_io.image_read('/Users/ro/U/[ Asistencia ] - Proyecto de Investigacion/Source_Images/mdb155.pgm'))
--
-- SELECT * from pgcv_core.uri_base64(pgcv_io.image_read('/Users/ro/U/[ Asistencia ] - Proyecto de Investigacion/Source_Images/mdb155.pgm'))DROP SCHEMA IF EXISTS  pgcv_io CASCADE;
CREATE SCHEMA pgcv_io;

-- =============================================
-- Author:      Roberto Mora
-- Description: Reads an image from a file into an ndarray_int4
-- =============================================
CREATE OR REPLACE FUNCTION pgcv_io.image_read(filename varchar)
  RETURNS pgcv_core.ndarray_int4
AS $$
"""
Reads an image from a file into an ndarray_int4

Parameters
----------
filename : str
    Filename of the image (path).
Returns
-------
image : ndarray
    The image represented by a pgcv_core.ndarray_int4.
Examples
--------
>>> SELECT shape FROM pgcv_io.image_read('/path/to/image.png');
"""

import numpy as np
from PIL import Image

img = Image.open(filename)
img = np.array(img)

return (list(img.shape), np.ravel(img))
$$ LANGUAGE plpython3u STRICT;
COMMENT ON FUNCTION pgcv_io.image_read(varchar) IS 'Reads an image from a file into an ndarray_int4.';

-- =============================================
-- Author:      Roberto Mora
-- Description: Writes an image from an ndarray_int4 into the specified filename (path)
-- =============================================
CREATE OR REPLACE FUNCTION pgcv_io.image_write(image pgcv_core.ndarray_int4, filename varchar)
  RETURNS boolean
AS $$
"""
Writes an image from an ndarray_int4 into the specified filename (path)

Parameters
----------
image : ndarray
    The image represented by a pgcv_core.ndarray_int4.
filename : str
    Filename of the image (path)
Returns
-------
success : bool
    True if the image was saved successfully
Examples
--------
>>> DO $_$
>>> DECLARE arr pgcv_core.ndarray_int4;
>>> BEGIN
>>> SELECT shape, data FROM pgcv_io.image_read('/path/to/in_image.png') INTO arr;
>>> PERFORM pgcv_io.image_write(arr, '/path/to/out_image.png');
>>> END
>>> $_$
"""

import numpy as np
from PIL import Image

img = np.array(image["data"]).reshape(image["shape"]).astype('uint8')
img = Image.fromarray(img)
img.save(filename)

return True
$$ LANGUAGE plpython3u STRICT;
COMMENT ON FUNCTION pgcv_io.image_write(pgcv_core.ndarray_int4, varchar) IS 'Writes an image from an ndarray_int4 into the specified filename (path).';


-- =============================================
-- These queries test the IO functions
-- =============================================
-- SELECT shape FROM pgcv_io.image_read('/Users/ro/U/[ Asistencia ] - Proyecto de Investigacion/Source_Images/mdb155.pgm');
--
-- DO $_$
-- DECLARE arr pgcv_core.ndarray_int4;
-- BEGIN
-- SELECT shape, data FROM pgcv_io.image_read('/Users/ro/U/[ Asistencia ] - Proyecto de Investigacion/Source_Images/mdb155.pgm') INTO arr;
-- PERFORM pgcv_io.image_write(arr, '/Users/ro/Desktop/pba.png');
-- END
-- $_$
DROP SCHEMA IF EXISTS  pgcv_filter CASCADE;
CREATE SCHEMA pgcv_filter;

-- =============================================
-- Author:      Roberto Mora
-- Description: Perform a median filter on an N-dimensional array.
-- =============================================
CREATE OR REPLACE FUNCTION pgcv_filter.blur_median(image pgcv_core.ndarray_int4, kernel int DEFAULT 5)
  RETURNS pgcv_core.ndarray_int4
AS $$
"""
Perform a median filter on an N-dimensional array.

Apply a median filter to the input array using a local window-size given by kernel.

Parameters
----------
image : ndarray
    The image represented by a pgcv_core.ndarray_int4.
kernel : int, optional
    Local window-size giving the size of the median filter. Defaults to 5
Returns
-------
image : ndarray
    The image represented by a pgcv_core.ndarray_int4.
Examples
--------
>>> DO $_$
>>> DECLARE
>>>   arr pgcv_core.ndarray_int4;
>>>   arr2 pgcv_core.ndarray_int4;
>>> BEGIN
>>> SELECT shape, data FROM pgcv_io.image_read('/path/to/image.png') INTO arr;
>>> SELECT shape, data FROM pgcv_filter.blur_median(arr, 4) INTO arr2;
>>> PERFORM pgcv_io.image_write(arr, '/path/to/unmodified_image.png');
>>> PERFORM pgcv_io.image_write(arr2, '/path/to/modified_image.png');
>>> END
>>> $_$
Notes
-----
Kernel size must be odd
"""

import numpy as np
from scipy import signal

global kernel

if kernel % 2 == 0:
  plpy.notice('Invalid kernel size "{}". Kernel must be odd. Default size 5 will be used instead'.format(kernel))
  kernel = 5

img = np.array(image["data"]).reshape(image["shape"]).astype('uint8')
med = signal.medfilt(img, kernel).astype('uint8')

return (list(med.shape), np.ravel(med))
$$ LANGUAGE plpython3u STRICT;
COMMENT ON FUNCTION pgcv_filter.blur_median(pgcv_core.ndarray_int4, int) IS 'Perform a median filter on an N-dimensional array.';

-- =============================================
-- Author:      Roberto Mora
-- Description: Calculates a threshold value based on Otsu's method.
-- =============================================
CREATE OR REPLACE FUNCTION pgcv_filter.threshold_otsu(image pgcv_core.ndarray_int4)
  RETURNS float
AS $$
"""
Calculates a threshold value based on Otsu's method.

Parameters
----------
image : ndarray
    The image represented by a pgcv_core.ndarray_int4.
Returns
-------
thresh : float
    A float that is the Otsu's threshold.
Examples
--------
>>> SELECT pgcv_filter.threshold_otsu(pgcv_io.image_read('/path/to/image.png'));
"""

import numpy as np
from skimage import filters

img = np.array(image["data"]).reshape(image["shape"]).astype('uint8')
thresh = filters.threshold_otsu(img)

return thresh
$$ LANGUAGE plpython3u STRICT;
COMMENT ON FUNCTION pgcv_filter.threshold_otsu(pgcv_core.ndarray_int4) IS 'Calculates a threshold value based on Otsu''s method.';

-- =============================================
-- Author:      Roberto Mora
-- Description: Enhances an image using the otsu threshold. Used for mammogram analysis
-- =============================================
CREATE OR REPLACE FUNCTION pgcv_filter.enhancement_otsu(image pgcv_core.ndarray_int4)
  RETURNS pgcv_core.ndarray_int4
AS $$
"""
Enhances an image using the otsu threshold. Used for mammogram analysis.

This method was designed by Johnny Villalobos and is described as:
Let t be the threshold of an image calculated through the Otsu's method and f the enhancement factor so that f = t / (255 - t),
the value of each enhanced pixel corresponds to:
    p'  = (255 - p(1 + f)) - f(255 - p(1 + f))
        = (1 - f) (255 - p(1 + f))

This formula has proven to be quite effective for mammogram segmentation

Parameters
----------
image : ndarray
    The image represented by a pgcv_core.ndarray_int4.
Returns
-------
image : ndarray
    Enhanced image represented by a pgcv_core.ndarray_int4.
Examples
--------
>>> SELECT shape from pgcv_filter.enhancement_otsu(pgcv_io.image_read('/path/to/image.png'));
>>> SELECT * from pgcv_io.image_write(
>>>     pgcv_filter.enhancement_otsu(
>>>         pgcv_filter.blur_median(
>>>             pgcv_io.image_read('/path/to/in_image.png'), 5
>>>         )
>>>     ), '/path/to/out_image.png');
Notes
-----
The median blur should be applied previous to this enhancement
"""

import numpy as np
from skimage import filters

img = np.array(image["data"]).reshape(image["shape"]).astype('uint8')
thresh = filters.threshold_otsu(img)
f = thresh / (255 - thresh)

img = (1 - f) * (255 - img * (1 + f))

return (list(img.shape), np.ravel(img).astype('uint8'))
$$ LANGUAGE plpython3u STRICT;
COMMENT ON FUNCTION pgcv_filter.enhancement_otsu(pgcv_core.ndarray_int4) IS 'Enhances an image using the otsu threshold. Used for mammogram analysis';

-- =============================================
-- Author:      Roberto Mora
-- Description: Binarizes an image according to the supplied threshold.
-- =============================================
CREATE OR REPLACE FUNCTION pgcv_filter.binarize(image pgcv_core.ndarray_int4, thresh float DEFAULT 255)
  RETURNS pgcv_core.ndarray_int4
AS $$
"""
Binarizes an image according to the supplied threshold.

Parameters
----------
image : ndarray
    The image represented by a pgcv_core.ndarray_int4.
thresh : float
    The threshold used to binarize the image. The resulting value is 0 if the value is less than the threshold, 255 otherwise
Returns
-------
image : ndarray
    Binarized image represented by a pgcv_core.ndarray_int4.
Examples
--------
>>> SELECT shape from pgcv_filter.binarize(pgcv_io.image_read('/path/to/image.png'), 77);
Notes
-----
If used for mammogram segmentation, consider to apply the enhancement_otsu first and use the threshold_otsu as threshold
"""

import numpy as np

img = np.array(image["data"]).reshape(image["shape"]).astype('uint8')
img = np.where(img < thresh, 0, 255)

return (list(img.shape), np.ravel(img).astype('uint8'))
$$ LANGUAGE plpython3u STRICT;
COMMENT ON FUNCTION pgcv_filter.binarize(pgcv_core.ndarray_int4, float) IS 'Binarizes an image according to the supplied threshold.';

-- =============================================
-- These queries test the filtering functions
-- =============================================
-- SELECT * from pgcv_io.image_write(
--     pgcv_filter.enhancement_otsu(
--         pgcv_filter.blur_median(
--             pgcv_io.image_read('/Users/ro/U/[ Asistencia ] - Proyecto de Investigacion/Source_Images/mdb155.pgm'), 5
--         )
--     ), '/Users/ro/Desktop/pba.png');
--
-- DO $_$
-- DECLARE
--   image pgcv_core.ndarray_int4;
--   result pgcv_core.ndarray_int4;
--   thresh float;
-- BEGIN
--   image :=  (SELECT pgcv_filter.blur_median(
--                       pgcv_io.image_read('/Users/ro/Desktop/mdb147lx.pgm'), 5
--                     ));
--   thresh := (SELECT pgcv_filter.threshold_otsu(image));
--   result := (SELECT pgcv_filter.binarize(pgcv_filter.enhancement_otsu(image), thresh));
--   PERFORM pgcv_io.image_write(result, '/Users/ro/Desktop/prueba.png');
-- END
-- $_$
DROP SCHEMA IF EXISTS  pgcv_histogram CASCADE;
CREATE SCHEMA pgcv_histogram;

-- =============================================
-- Author:      Roberto Mora
-- Description: Compute the histogram of a set of data and the bin edges
-- =============================================
CREATE OR REPLACE FUNCTION pgcv_histogram.hist_bin_edges(image pgcv_core.ndarray_int4, bins int DEFAULT 10, as_float boolean DEFAULT TRUE,
  OUT hist numeric[], OUT bin_edges numeric[])
AS $$
"""
Compute the histogram of a set of data and the bin edges

Parameters
----------
image : ndarray
    The image represented by a pgcv_core.ndarray_int4.
bins : int, optional
    Number of bins in the histogram. Defaults to 10
as_float : bool, optional
    Indicates if the image should be converted as float for the histogram computation. Defaults to TRUE
Returns
-------
hist : array
    The histogram represented by an numeric array.
bin_edges : array
    The bin_edges represented by a numeric array.
Examples
--------
>>> SELECT * FROM pgcv_histogram.hist_bin_edges(pgcv_io.image_read('/path/to/image.png'));
>>> SELECT * FROM pgcv_histogram.hist_bin_edges(pgcv_io.image_read('/path/to/image.png'), 5, FALSE);
>>> SELECT * FROM pgcv_histogram.hist_bin_edges(pgcv_io.image_read('/path/to/image.png'), 5);
Notes
-----
The bin_edges array is of length (length(hist) + 1)
"""

import numpy as np
from skimage import img_as_float

img = np.array(image["data"]).reshape(image["shape"]).astype('uint8')
if (as_float == True):
  img = img_as_float(img)
hist, bin_edges = np.histogram(img, bins)

return (hist, bin_edges)
$$ LANGUAGE plpython3u STRICT;
COMMENT ON FUNCTION pgcv_histogram.hist_bin_edges(pgcv_core.ndarray_int4, int, boolean, OUT int[], OUT numeric[]) IS 'Compute the histogram of a set of data and the bin edges. The user can choose to convert the image to float';

-- =============================================
-- Author:      Roberto Mora
-- Description: Compute the normalized histogram of a set of data and the bin centers
-- =============================================
CREATE OR REPLACE FUNCTION pgcv_histogram.hist_bin_centers(image pgcv_core.ndarray_int4, bins int DEFAULT 10,
  OUT hist numeric[], OUT bin_centers numeric[])
AS $$
"""
Compute the normalized histogram of a set of data and the bin centers

Parameters
----------
image : ndarray
    The image represented by a pgcv_core.ndarray_int4.
bins : int, optional
    Number of bins in the histogram. Defaults to 10
Returns
-------
hist : array
    The normalized histogram represented by an numeric array.
bin_centers : array
    The bin_centers represented by a numeric array.
Examples
--------
>>> SELECT * FROM pgcv_histogram.hist_bin_centers(pgcv_io.image_read('/path/to/image.png'), 5);
>>> SELECT * FROM pgcv_histogram.hist_bin_centers(pgcv_io.image_read('/path/to/image.png'));
Notes
-----
The bin_centers array is of length length(hist)
"""

import numpy as np
from skimage import exposure, img_as_float

img = np.array(image["data"]).reshape(image["shape"]).astype('uint8')
img = img_as_float(img)
hist, bin_centers = exposure.histogram(img, bins)

return (hist, bin_centers)
$$ LANGUAGE plpython3u STRICT;
COMMENT ON FUNCTION pgcv_histogram.hist_bin_centers(pgcv_core.ndarray_int4, int, OUT int[], OUT numeric[]) IS 'Compute the normalized histogram of a set of data and the bin centers.';


-- =============================================
-- These queries test the histogram functions
-- =============================================
-- SELECT * FROM pgcv_histogram.hist_bin_edges(pgcv_io.image_read('/Users/ro/U/[ Asistencia ] - Proyecto de Investigacion/Source_Images/mdb155.pgm'));
-- SELECT * FROM pgcv_histogram.hist_bin_edges(pgcv_io.image_read('/Users/ro/U/[ Asistencia ] - Proyecto de Investigacion/Source_Images/mdb155.pgm'), 5, FALSE);
-- SELECT * FROM pgcv_histogram.hist_bin_edges(pgcv_io.image_read('/Users/ro/U/[ Asistencia ] - Proyecto de Investigacion/Source_Images/mdb155.pgm'), 5);
--
-- SELECT * FROM pgcv_histogram.hist_bin_centers(pgcv_io.image_read('/Users/ro/U/[ Asistencia ] - Proyecto de Investigacion/Source_Images/mdb155.pgm'), 5);
-- SELECT * FROM pgcv_histogram.hist_bin_centers(pgcv_io.image_read('/Users/ro/U/[ Asistencia ] - Proyecto de Investigacion/Source_Images/mdb155.pgm'));
DROP SCHEMA IF EXISTS pgcv_measure CASCADE;
CREATE SCHEMA pgcv_measure;

-- =============================================
-- Author:      Roberto Mora
-- Description: Returns a json array with the region properties found in a binary image
-- =============================================
CREATE OR REPLACE FUNCTION pgcv_measure.region_props_json(image pgcv_core.ndarray_int4)
  RETURNS jsonb
AS $$
"""
Returns a json array with the region properties of a binary image

Parameters
----------
image : ndarray
    The image represented by a pgcv_core.ndarray_int4.
Returns
-------
regionprops : jsonb
    The json array with the region properties, corresponding to label, area, perimeter,
    centroid, solidity, eccentricity, convex_area, circularity, orientation and bbox.
Examples
--------
>>> SELECT pgcv_measure.region_props_json(pgcv_io.image_read('/path/to/binarized/image.png'));
Notes
-----
The input image must be binarized
"""

import numpy as np
import pandas as pd
from skimage import measure
import json

img = np.array(image["data"]).reshape(image["shape"]).astype('uint8')

# Get the regionprops
lbl = measure.label(img)
regions = measure.regionprops(lbl, coordinates='rc')

# Initialize the DataFrame with the desired columns
columns = [('label', int),
           ('area', int),
           ('perimeter', float),
           ('centroid', object),
           ('solidity', float),
           ('eccentricity', float),
           ('convex_area', int),
           ('circularity', float),
           ('orientation', float),
           ('bbox', object)]
df = pd.DataFrame({k: pd.Series(dtype=t) for k, t in columns})

# Fill the DataFrame
for i, reg in enumerate(regions):
    df.loc[i] = [
        reg.label,
        reg.area,
        reg.perimeter,
        reg.centroid,
        reg.solidity,
        reg.eccentricity,
        reg.convex_area,
        4 * np.pi * reg.area / reg.perimeter ** 2 if reg.perimeter != 0 else np.inf,  # circularity
        reg.orientation,
        reg.bbox
    ]

result = df.to_json(orient='records', double_precision=4)

return (result)
$$ LANGUAGE plpython3u STRICT;
COMMENT ON FUNCTION pgcv_measure.region_props_json(pgcv_core.ndarray_int4) IS 'Returns a json array with the region properties found in a binary image.';

-- =============================================
-- Author:      Roberto Mora
-- Description: Returns a set of region properties found in a binary image
-- =============================================
CREATE OR REPLACE FUNCTION pgcv_measure.region_props(image pgcv_core.ndarray_int4)
  RETURNS SETOF pgcv_core.regionprops
AS $$
"""
Returns a set of region properties found in a binary image

Parameters
----------
image : ndarray
    The image represented by a pgcv_core.ndarray_int4.
Returns
-------
regionprops : setof regionprops
    The set with the region properties, corresponding to label, area, perimeter,
    centroid, solidity, eccentricity, convex_area, circularity, orientation and bbox.
Examples
--------
>>> SELECT * FROM pgcv_measure.region_props(pgcv_io.image_read('/Users/ro/Desktop/prueba.png'))
>>>   WHERE area > 15 AND area < 55;
Notes
-----
The input image must be binarized
"""

import numpy as np
from skimage import measure

img = np.array(image["data"]).reshape(image["shape"]).astype('uint8')

# Get the regionprops
lbl = measure.label(img)
regions = measure.regionprops(lbl, coordinates='rc')

# Yield the results
for i, reg in enumerate(regions):
    yield (
        reg.label,
        reg.area,
        reg.perimeter,
        reg.centroid,
        reg.solidity,
        reg.eccentricity,
        reg.convex_area,
        4 * np.pi * reg.area / reg.perimeter ** 2 if reg.perimeter != 0 else np.inf,  # circularity
        reg.orientation,
        reg.bbox
    )
$$ LANGUAGE plpython3u STRICT;
COMMENT ON FUNCTION pgcv_measure.region_props(pgcv_core.ndarray_int4) IS 'Returns a set of region properties found in a binary image.';


-- =============================================
-- These queries test the measure functions
-- =============================================
-- SELECT pgcv_measure.region_props_json(pgcv_io.image_read('/Users/ro/Desktop/prueba.png'))::json->2 as props;
--
-- SELECT pgcv_measure.region_props_json(pgcv_io.image_read('/Users/ro/Desktop/prueba.png'));
--
-- SELECT * FROM pgcv_measure.region_props(pgcv_io.image_read('/Users/ro/Desktop/prueba.png')) WHERE area > 15 AND area < 55;
DROP SCHEMA IF EXISTS pgcv_bundle CASCADE;
CREATE SCHEMA pgcv_bundle;

-- =============================================
-- Author:      Roberto Mora
-- Description: Returns a set of region properties found in a mammogram image
-- =============================================
CREATE OR REPLACE FUNCTION pgcv_bundle.mam_region_props(image pgcv_core.ndarray_int4, kernel int DEFAULT 5)
  RETURNS SETOF pgcv_core.regionprops
AS $$
"""
Returns a set of region properties found in a mammogram image

Parameters
----------
image : ndarray
    The image represented by a pgcv_core.ndarray_int4.
Returns
-------
regionprops : setof regionprops
    The set with the region properties, corresponding to label, area, perimeter,
    centroid, solidity, eccentricity, convex_area, circularity, orientation and bbox.
Examples
--------
>>> SELECT * FROM pgcv_bundle.mam_region_props(pgcv_io.image_read('path/to/image.png'))
>>>   WHERE area > 15 AND area < 55;
Notes
-----
The input image must be unaltered.
"""

import numpy as np
import pandas as pd
from scipy import signal
from skimage import measure, filters

img = np.array(image["data"]).reshape(image["shape"]).astype('uint8')

"""
1. Apply the mean filter
"""
img = signal.medfilt(img, kernel).astype('uint8')

"""
2. Enhance the image through the otsu enhancement
"""
thresh = filters.threshold_otsu(img)
f = thresh / (255 - thresh)
img = ((1 - f) * (255 - img * (1 + f))).astype('uint8')

"""
3. Binarize the image
"""
img = np.where(img < thresh, 0, 255)

"""
4. Calculate the regionprops
"""
# Get the regionprops
lbl = measure.label(img)
regions = measure.regionprops(lbl, coordinates='rc')

# Yield the results
for i, reg in enumerate(regions):
    yield (
        reg.label,
        reg.area,
        reg.perimeter,
        reg.centroid,
        reg.solidity,
        reg.eccentricity,
        reg.convex_area,
        4 * np.pi * reg.area / reg.perimeter ** 2 if reg.perimeter != 0 else np.inf,  # circularity
        reg.orientation,
        reg.bbox
    )
$$ LANGUAGE plpython3u STRICT;
COMMENT ON FUNCTION pgcv_bundle.mam_region_props(pgcv_core.ndarray_int4, int) IS 'Returns a set of region properties found in a mammogram image';

-- =============================================
-- Author:      Roberto Mora
-- Description: Returns an image segmented through Otsu's enhancement
-- =============================================
CREATE OR REPLACE FUNCTION pgcv_bundle.mam_segment(image pgcv_core.ndarray_int4, kernel int DEFAULT 5)
  RETURNS pgcv_core.ndarray_int4
AS $$
"""
Returns an image segmented through Otsu's enhancement

Parameters
----------
image : ndarray
    The image represented by a pgcv_core.ndarray_int4.
Returns
-------
segmented_image : ndarray
    An image segmented through Otsu's enhancement
Examples
--------
>>> SELECT * FROM pgcv_bundle.mam_segment(pgcv_io.image_read('/path/to/image.png'));
"""

import numpy as np
from scipy import signal
from skimage import filters

img = np.array(image["data"]).reshape(image["shape"]).astype('uint8')

"""
1. Apply the mean filter
"""
img = signal.medfilt(img, kernel).astype('uint8')

"""
2. Enhance the image through the otsu enhancement
"""
thresh = filters.threshold_otsu(img)
f = thresh / (255 - thresh)
img = ((1 - f) * (255 - img * (1 + f))).astype('uint8')

"""
3. Binarize the image
"""
img = np.where(img < thresh, 0, 255)

"""
4. Return the segmented image
"""

return (list(img.shape), np.ravel(img).astype('uint8'))
$$ LANGUAGE plpython3u STRICT;
COMMENT ON FUNCTION pgcv_bundle.mam_segment(pgcv_core.ndarray_int4, int) IS 'Returns an image segmented through Otsu''s enhancement';


-- =============================================
-- These queries test the bundle functions
-- =============================================
-- SELECT * FROM pgcv_bundle.mam_region_props(pgcv_io.image_read('/Users/ro/U/[ Asistencia ] - Proyecto de Investigacion/Source_Images/mdb155.pgm')) WHERE area > 15 AND area < 55;
-- SELECT * FROM pgcv_bundle.mam_segment(pgcv_io.image_read('/Users/ro/U/[ Asistencia ] - Proyecto de Investigacion/Source_Images/mdb155.pgm'));
